
/********************************************************************************************************************
All ADuC706x library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

Status:			Tested in simulation.
Tools:			RealView compiler.
Hardware:		ADuC7060/61/62.
Description:	This file demonstrates how to generate a 3-Phase PWM output where each output
				signal is 120Degrees phase shifted.
				P3.0 = Phase1 control
				P2.0 = Phase2 control
				P2.1 = Phase3 control
Note: IRQ_ARM.C should not be built with this file!

Responsibility:
	Group:		IAC Apps.
	Person:		
Modifications:
	2008/10/22	Design Start.
************************************************************************************************************************/


#include<aduc7026.h>

volatile unsigned char ucFirstPWMedge = 1;
volatile unsigned char ucPhase = 1;
void IRQ_Handler(void) __irq;		//	IRQ Funtion Prototype 
volatile unsigned int uiFIQSTA = 0;
void delay(int TIME123)
{
	while(TIME123>0)
	{
		TIME123--;
	}
return;
}
void IRQ_Handler(void) __irq
{
   unsigned long ulIRQStatus = 0;
   
   ulIRQStatus = IRQSTA;	// Read IRQSTA register
   PWMSTA = 0x100;
   if ((ulIRQStatus & 0x0800000) == 0x800000)	// PWM IRQ
   {
   		PWMSTA = 0x100;
   }
}
void FIQ_Handler(void) __irq
{
   unsigned long ulFIQStatus = 0;

   ulFIQStatus = FIQSTA;	// Read IRQSTA register
   PWMSTA = 0x200;
   if ((ulFIQStatus & 0x0800000) == 0x800000)	// PWM IRQ
   {
   		PWMSTA = 0x100;
   }
   if ((ulFIQStatus & 0x0100000) == 0x100000)	// PLA1 IRQ
   { 		 
		if (ucFirstPWMedge == 1) //Kick off Phase 2 and Phase 3 timers here
		{
			T1CON = 0xC0;		// Enable timer 1 in periodic mode, HCLK source
			T1LD = 0xA9EF;		// 8.33mS period to first Phase 2 IRQ
			GP3DAT ^=0x40000;	// Toggle P3.2
			ucPhase = 0x1;

			T0CON = 0xC0;
			T0LD = 0x7148;		// 5.55mS period
			ucFirstPWMedge = 0; // Clear First PWM IRQ flag
   			FIQCLR = 0x0100000;	// Clear PLA interrupt source
			FIQEN |= 0xC;		// Enable Timer0, 1 and 2	IRQ sources
		}
		else
		{

		}				
   }
   if ((ulFIQStatus & 0x4) == 0x4) // timer 0 interrupt source
   {
			GP3DAT ^=0x80000;	// Toggle P3.3
			T0CLRI = 0xFF;		// Clear the current Timer 1 IRQ
			FIQCLR = 0x4;		// Clear Timer 0 FIQ source
   }
   if ((ulFIQStatus & 0x8) == 0x8) // timer 1 interrupt source
   {
		T1CLRI = 0xFF;		// Clear the current Timer 1 IRQ
		switch (ucPhase)		// 8.33mS period
		{ 
			case 1:	// Phase 1
			{
				GP3DAT ^=0x40000;	// Toggle P3.2
				T1LD = 0x38EF;		// 8.33mS period
				ucPhase = 3;		// Setup next phase
				break;
			}
			case 2:	// Phase 2
			{
				GP3DAT ^=0x80000;	// Toggle P3.3
				T1LD = 0x38EF;		// 8.33mS period
				ucPhase = 1;		// Setup next phase
				break;
			}
			case 3:	// Phase 3
			{
			   GP3DAT ^=0x100000;	// Toggle P3.4 
			   T1LD = 0x38EF;		// 8.33mS period
			   ucPhase = 2;		// Setup next phase
			   break;
			}
			default: // Error condition!
			{
				GP3DAT ^=0x40000;	// Toggle P3.2
				T1LD = 0xA9EF;		// 8.33mS period
				ucPhase = 1;		// Setup next phase
				break;
			}
		}
		
   }
   if ((ulFIQStatus & 0x10) == 0x10) // timer 2 interrupt source
   {
			GP3DAT ^=0x80000;	// Toggle P3.3
			T2LD = 0xA9EF;		// 8.33mS period 
			T2CLRI = 0xFF;		// Clear the current Timer 1 IRQ
			FIQCLR = 0x10;		// Clear Timer 2 FIQ source				
   }
   return;
}


int main(void) 
{
   POWKEY1 = 0x01;	   		// Set core frequency to 5.22Mhz
   POWCON = 0x03;
   POWKEY2 = 0xF4;
   
   ucFirstPWMedge = 1;		// Flagged to indicate first PLA IRQ
   // Configure GPIO pins
   GP4DAT = 0xffff0000;  	
//   GP3CON = 0x11100011;		// Enable the PWM outputs to the GPIO
      
   GP3CON = 0x11111111;
   GP0CON = 0x110;			// Enable the PWM outputs to the GPIO
   GP3DAT = 0x1C000000;		// Configure P3.2/P3.3/P3.4 as PWM phase outputs/GPIO outputs
   							// Controlled by Timer 1 and Timer 2

   // Configure PLA element 8 - will kick off Phase 2/3 timers

   PLAELM8 = 0x34;			// Select P3.0 as input to Flip-flop of element 8
   PLACLK = 0x30;			// Select P0.7 as F-flop clock source
   							// Connect P3.0 to P0.7 externally
   PLAIRQ = 0x1800;			// Enable PLA IRQ
   
   // Setup the PWM	
   PWMEN = 0x100;			// Switch Channel 0 outputs
   PWMCON = 0x13;   			// Ext ASYNC enabled
 //  PWMCON = 0x1;   			// 0x01 is enabled
   PWMDAT0 = 0xA400;  		// Period register 15.8mS
   PWMDAT1 = 0x3FF;    		// Dead time

   PWMCH0 = 0x500;   		// duty cycle channel 0
   PWMCH1 = 0x4000;      	// duty cycle channel 1
   PWMCH2= 0xE000;      	// duty cycle channel 2


   
   // Enable PLA FIQ
//   FIQEN = 0x0100000;		// Enable PLA1 IRQ.
	FIQEN = 0x900000;		// Enable PLA1 IRQ.
//	IRQEN = 0x800000;
   while (1)
   {
   }

}
